@@ -29,8 +29,9 @@
 
 from __future__ import print_function
 
+import os
 import operator
-from functools import wraps
+from functools import partial, wraps
 from itertools import starmap, permutations
 import sys
 
@@ -76,6 +77,33 @@ def _RangeCaster(value):
         raise_error()
 
 
+class _DirectoryOpenerCaster(object):
+    def __init__(self, create):
+        self._create = create
+
+    def __call__(self, name):
+        if not os.path.exists(name):
+            if self._create:
+                os.makedirs(name)
+                return name
+            raise IOError('%s does not exist' % name)
+
+        if not os.path.isdir(name):
+            raise IOError('%s is not directory' % name)
+
+        return name
+
+
+def _FileOpenerCaster(mode=None, buffering=None):
+    kw = {}
+    if mode is not None:
+        kw['mode'] = mode
+    if buffering is not None:
+        kw['buffering'] = buffering
+
+    return partial(open, **kw)
+
+
 # ---------- decorators ---------- #
 
 
@@ -337,45 +365,28 @@ class Option(Condition):
         self._parser = parser
         self._conditions = []
 
-    def requires(self, *others):
-        ''' Specifiy other options which this argument requires.
+    def requires(self, *conditions):
+        ''' Specifiy other options/conditions which this argument requires.
 
-        :param others: required options
-        :type others: sequence of either :class:`Option` or basestring of \
-                    option names
+        :param conditions: required conditions
+        :type others: sequence of either :class:`Option` or :class:`Condition`s 
         '''
-        [self._parser._set_requires(self.argname, x) for x in others]
+        [self._parser._set_requires(self.argname, x) for x in conditions]
         return self
 
     def condition(self, func):
         self._conditions.append(func)
         return self
 
-    def conflicts(self, *others):
-        ''' Specifiy other options which this argument conflicts with.
+    def conflicts(self, *conditions):
+        ''' Specifiy other conditions which this argument conflicts with.
 
-        :param others: conflicting options
-        :type others: sequence of either :class:`Option` or basestring of \
-            option names
+        :param conditions: conflicting options/conditions
+        :type conditions: sequence of either :class:`Option` or :class:`Condition`
         '''
-        [self._parser._set_conflicts(self.argname, x) for x in others]
+        [self._parser._set_conflicts(self.argname, x) for x in conditions]
         return self
 
-    def _cast(self):
-        return self._parser._casts.get(self.argname)
-
-    def _isrequired(self):
-        return self in self._parser._required
-
-    def _getconflicts(self):
-        return self._parser._conflicts.get(self, [])
-
-    def _getreqs(self):
-        return self._parser._requires.get(self, [])
-
-    def _alias(self):
-        return self._parser._source_to_alias.get(self.argname)
-
     def __str__(self):
         v = '%s%s' % (self._parser._double_prefix, self.argname)
         alias = self._alias()
@@ -436,6 +447,8 @@ class Option(Condition):
         self._parser._set_multiple(self.argname)
         return self
 
+    # --- conditions
+
     def _inner_satisfied(self, parsed):
         v = parsed.get(self.argname)
         return v is not None and v != False
@@ -461,6 +474,23 @@ class Option(Condition):
     def __ne__(self, other):
         return self._make_condition(operator.__ne__, other)
 
+    # -- private access methods
+
+    def _cast(self):
+        return self._parser._casts.get(self.argname)
+
+    def _isrequired(self):
+        return self in self._parser._required
+
+    def _getconflicts(self):
+        return self._parser._conflicts.get(self, [])
+
+    def _getreqs(self):
+        return self._parser._requires.get(self, [])
+
+    def _alias(self):
+        return self._parser._source_to_alias.get(self.argname)
+
 
 # ---------- Argument readers ---------- #
 
@@ -501,6 +531,8 @@ class _MultiWordArgumentReader(_ArgumentReader):
         return True
 
     def get(self):
+        if len(self.value) == 0:
+            raise MissingValueError
         return ' '.join(self.value)
 
 
@@ -563,6 +595,11 @@ class Group(Option):
         return ', '.join([str(name) for name in self._names])
 
 
+class FakeSystemExit(ValueError):
+    # Used for testing
+    pass
+
+
 class Parser(object):
     ''' Command line parser. '''
 
@@ -602,6 +639,7 @@ class Parser(object):
         # help message
         self._help_prefix = None
 
+        self._suppress_sys_exit = False
         self._out = sys.stdout
 
         # set by user
@@ -742,6 +780,42 @@ class Parser(object):
         self._set_reader(name, _FlagArgumentReader)
         return result
 
+    def file(self, name, mode=None, buffering=None):
+        ''' Opens the file indicated by the name passed by the user. ``mode``
+        and ``buffering`` are arguments passed to ``open``.
+
+        The example below implements a file copy operation:
+
+        >>> with Parser(locals()) as p:
+        ...     p.file('input_file')
+        ...     p.file('output_file', mode='w')
+        ...
+        ... output_file.write(input_file.read())
+        
+        '''
+
+        return self.multiword(name).cast(_FileOpenerCaster(mode, buffering))
+
+    def directory(self, name, create=False):
+        ''' File directory value. Checks to ensure that the user passed file
+        name exists and is a directory (i.e., not some other file object). If
+        ``create`` is specified, creates the directory using ``os.makedirs``;
+        any intermediate directories are also created. '''
+
+        return self.multiword(name).cast(_DirectoryOpenerCaster(create))
+
+    def url(self, name):
+        ''' URL value; verifies that argument has a scheme (e.g., http, ftp,
+        file). '''
+
+        from urlparse import urlparse
+        def parse(value):
+            if urlparse(value).scheme == '':
+                raise FormatError('%s not valid URL' % value)
+            return value
+
+        self.str(name).cast(parse)
+
 # --- aggregate calls --- #
 
     def at_least_one(self, *args):
@@ -1071,6 +1145,8 @@ class Parser(object):
         msg.append('usage: %s' % sys.argv[0])
         msg.append(' '.join('[%s]' % self._label(value) for value in self._options.values()))
         self.emit('\n'.join(msg))
+        if self._suppress_sys_exit:
+            raise FakeSystemExit
         sys.exit(1)
 
     @localize
@@ -1166,53 +1242,5 @@ class Parser(object):
             self.emit(msg)
 
 
-class IOParser(Parser):
-    def directory(self, name):
-        ''' Adds a directory option, as :class:`io.Directory` object. Requires
-        directory to exist or raises exception. '''
-
-        from plyny.plio.files import Directory
-        return self._add_option(name).cast(Directory)
-
-    def file(self, name):
-        ''' Adds a :class:`io.File` option. '''
-        from plyny.plio.files import open_path
-        return self._add_option(name).cast(open_path)
-
-    def input_file(self, name):
-        ''' Adds an input file option, as :class:`io.File` object. Requires
-        file to exist or raises exception. '''
-
-        from plyny.plio.files import open_path
-
-        def opener(x):
-            try:
-                f = open_path(x, True)
-#            f.set_locked(True)
-            except Exception as e:
-                print(e)
-            return f
-
-        result = self._add_option(name).cast(opener)
-        self._set_reader(name, _MultiWordArgumentReader)
-        return result
-
-    def output_file(self, name, disable_overwrite=True):
-        ''' Adds an output file option, as :class:`io.FileWriter` object.
-        Prevents the overwriting of the file.  '''
-
-        from plyny.plio.files import stdio, FileWriter
-
-        name = self._localize(name)
-
-        def opener(x):
-            if x == '-':
-                return stdio()
-            else:
-                return FileWriter(x, not disable_overwrite)
-
-        return self._add_option(name).cast(opener)
-
-
 __all__ = ['Parser']
-__version__ = '0.2.14a'
+__version__ = '0.2.17a'
